import axios, { AxiosResponse } from "axios";
import {
  InterestRatesQueryRequest,
  InterestRatesResponse,
  LoanToValueQueryRequest,
  LoanToValueResponse,
} from "./schemas/information.schema";
import {
  CreateAccountQueryRequest,
  CreateAccountRequest,
  ReserveAccountQueryRequest,
  ReserveAccountResponse,
} from "./schemas/accounts.schema";

/**
 * Reserve account
 * @param baseUrl - Baseurl to service url @defaults to http://services-core
 */
export async function reserveAccount(
  query: ReserveAccountQueryRequest,
  baseUrl: string = "http://services-core"
): Promise<ReserveAccountResponse> {
  const response = await axios.post<any, AxiosResponse<ReserveAccountResponse>>(
    `${baseUrl}/api/accounts/reservation`,
    null,
    { params: query }
  );
  return response.data;
}

/**
 * Create account
 * @param baseUrl - Baseurl to service url @defaults to http://services-core
 */
export async function createAccount(
  query: CreateAccountQueryRequest,
  data: CreateAccountRequest,
  baseUrl: string = "http://services-core"
): Promise<any> {
  const response = await axios.post<any, AxiosResponse<any>>(
    `${baseUrl}/api/accounts`,
    data,
    { params: query }
  );
  return response.data;
}

/**
 * Get interest rates
 * @param baseUrl - Baseurl to service url @defaults to http://services-core
 */
export async function getInterestRates(
  query: InterestRatesQueryRequest,
  baseUrl: string = "http://services-core"
): Promise<InterestRatesResponse> {
  const response = await axios.get<any, AxiosResponse<InterestRatesResponse>>(
    `${baseUrl}/api/information/interest-rates`,
    { params: query }
  );
  return response.data;
}

/**
 * Get loan to value
 * @param baseUrl - Baseurl to service url @defaults to http://services-core
 */
export async function getLoanToValue(
  query: LoanToValueQueryRequest,
  baseUrl: string = "http://services-core"
): Promise<LoanToValueResponse> {
  const response = await axios.get<any, AxiosResponse<LoanToValueResponse>>(
    `${baseUrl}/api/information/loan-to-value`,
    { params: query }
  );
  return response.data;
}

export type {
  CreateAccountRequest,
  CreateAccountQueryRequest,
  ReserveAccountQueryRequest,
  ReserveAccountResponse,
  InterestRatesQueryRequest,
  InterestRatesResponse,
  LoanToValueQueryRequest,
  LoanToValueResponse,
};
